@@ -173,7 +173,8 @@ class Module(AuxDataContainer):
         sections=set(),  # type: typing.Iterable[Section]
         symbols=set(),  # type: typing.Iterable[Symbol]
         entry_point=None,  # type: typing.Optional[CodeBlock]
-        uuid=None  # type: typing.Optional[UUID]
+        uuid=None,  # type: typing.Optional[UUID]
+        ir=None  # type: typing.Optional["IR"]
     ):
         # type: (...) -> None
         """
@@ -199,6 +200,7 @@ class Module(AuxDataContainer):
         :param uuid: The UUID of this ``Module``,
             or None if a new UUID needs generated via :func:`uuid.uuid4`.
             Defaults to None.
+        :param ir: The :class:`IR` this module belongs to.
         """
 
         self._ir = None  # type: "IR"
@@ -222,6 +224,9 @@ class Module(AuxDataContainer):
         # Initialize the aux data last so that the cache is populated
         super().__init__(aux_data, uuid)
 
+        # Use the property setter to ensure correct invariants.
+        self.ir = ir
+
     @classmethod
     def _decode_protobuf(cls, proto_module, uuid, ir):
         # type: (Module_pb2.Module. UUID, typing.Optiona["IR"]) -> Module
@@ -357,55 +362,6 @@ class Module(AuxDataContainer):
         if value is not None:
             value.modules.append(self)
 
-    @property
-    def address(self):
-        # type: () -> typing.Optional[int]
-        """Get the address of this module, if known.
-
-        The address is calculated from the :class:`Section` objects in
-        this module. More specifically, if the address of all sections
-        in this module are fixed, then it will return the address of the
-        section lowest in memory. If any one section does not have an address,
-        then this will be ``None``, as the address is not calculable in that
-        case. Note that a module with no sections in it has no address or
-        size, so it will be ``None`` in that case.
-        """
-
-        lowest = None
-        for s in self.sections:
-            if s.address is None:
-                return None
-            if lowest is None or s.address < lowest:
-                lowest = s.address
-        return lowest
-
-    @property
-    def size(self):
-        # type: () -> typing.Optional[int]
-        """Get the size of this module, if known.
-
-        The address is calculated from the :class:`Section` objects in
-        this module. More specifically, if the address of all sections
-        in this module are fixed, then it will return the difference between
-        the lowest and highest address among the sections. If any one section
-        does not have an address, then this will be ``None``, as the size is
-        not calculable in that case. Note that a module with no sections in
-        it has no address or size, so it will be ``None`` in that case.
-        """
-
-        lowest = None
-        highest = None
-        for s in self.sections:
-            if s.address is None:
-                return None
-            if lowest is None or s.address < lowest:
-                lowest = s.address
-            if highest is None or s.address + s.size > highest:
-                highest = s.address + s.size
-        if lowest is None or highest is None:
-            return None
-        return highest - lowest
-
     @property
     def byte_intervals(self):
         # type: () -> typing.Iterator[ByteInterval]
