@@ -33,8 +33,32 @@ class SymbolicExpression:
     def __init__(
         self, attributes=set(),
     ):
+        # type: ("AttributesCtorType") -> None
         self.attributes = set(attributes)
 
+    @property
+    def symbols(self):
+        # type: () -> typing.Iterable[Symbol]
+        """Get all the symbols involved with this symbolic expression,
+        regardless of role.
+        """
+
+        return ()
+
+    def _attributes_repr(self):
+        # type: () -> str
+        if not self.attributes:
+            return "set()"
+        else:
+            return "{%s}" % ",".join(
+                "SymbolicExpression.Attribute.%s" % a.name
+                for a in self.attributes
+            )
+
+
+if typing.TYPE_CHECKING:
+    AttributesCtorType = typing.Iterable[SymbolicExpression.Attribute]
+
 
 class SymAddrAddr(SymbolicExpression):
     """Represents a symbolic expression of the form
@@ -52,7 +76,7 @@ class SymAddrAddr(SymbolicExpression):
         offset,  # type: int
         symbol1,  # type: Symbol
         symbol2,  # type: Symbol
-        attributes=set(),
+        attributes=set(),  # type: AttributesCtorType
     ):
         # type: (...) -> None
         """
@@ -117,8 +141,9 @@ class SymAddrAddr(SymbolicExpression):
             "offset={offset!r}, "
             "symbol1={symbol1!r}, "
             "symbol2={symbol2!r}, "
-            ")".format(**self.__dict__)
-        )
+            "attributes={attributes_repr!s}, "
+            ")"
+        ).format(attributes_repr=self._attributes_repr(), **self.__dict__)
 
     def deep_eq(self, other):
         # type: (typing.Any) -> bool
@@ -133,6 +158,11 @@ class SymAddrAddr(SymbolicExpression):
             and self.attributes == other.attributes
         )
 
+    @property
+    def symbols(self):
+        yield self.symbol1
+        yield self.symbol2
+
 
 class SymAddrConst(SymbolicExpression):
     """Represents a symbolic expression of the form "Sym + Offset".
@@ -142,7 +172,7 @@ class SymAddrConst(SymbolicExpression):
     """
 
     def __init__(self, offset, symbol, attributes=set()):
-        # type: (int,Symbol) -> None
+        # type: (int, Symbol, AttributesCtorType) -> None
         """
         :param offset: Constant offset.
         :param symbol: Symbol representing an address.
@@ -191,8 +221,9 @@ class SymAddrConst(SymbolicExpression):
             "SymAddrConst("
             "offset={offset!r}, "
             "symbol={symbol!r}, "
-            ")".format(**self.__dict__)
-        )
+            "attributes={attributes_repr!s}, "
+            ")"
+        ).format(attributes_repr=self._attributes_repr(), **self.__dict__)
 
     def deep_eq(self, other):
         # type: (typing.Any) -> bool
@@ -205,76 +236,6 @@ class SymAddrConst(SymbolicExpression):
             and self.attributes == other.attributes
         )
 
-
-class SymStackConst(SymbolicExpression):
-    """Represents a symbolic expression of the form "Sym + Offset",
-    representing an offset from a stack variable.
-
-    :ivar ~.offset: Constant offset.
-    :ivar ~.symbol: Symbol representing a stack variable.
-    """
-
-    def __init__(self, offset, symbol, attributes=set()):
-        # type: (int,Symbol) -> None
-        """
-        :param offset: Constant offset.
-        :param symbol: Symbol representing a stack variable.
-        :param attributes: :class:`SymbolicExpression.Attribute`\\s of this
-            expression.
-        """
-
-        super().__init__(attributes)
-        self.offset = offset  # type: int
-        self.symbol = symbol  # type: Symbol
-
-    @classmethod
-    def _from_protobuf(
-        cls,
-        proto_symstackconst,  # type: SymbolicExpression_pb2.SymStackConst
-        get_by_uuid,  # type: typing.Callable[[UUID], Node]
-    ):
-        # type: (...) -> SymStackConst
-        symbol = get_by_uuid(UUID(bytes=proto_symstackconst.symbol_uuid))
-        return cls(proto_symstackconst.offset, symbol)
-
-    def _to_protobuf(self):
-        # type: () -> SymbolicExpression_pb2.SymStackConst
-        proto_symstackconst = SymbolicExpression_pb2.SymStackConst()
-        proto_symstackconst.offset = self.offset
-        if self.symbol is not None:
-            proto_symstackconst.symbol_uuid = self.symbol.uuid.bytes
-        return proto_symstackconst
-
-    def __eq__(self, other):
-        # type: (typing.Any) -> bool
-        if not isinstance(other, SymStackConst):
-            return False
-        return (
-            self.offset == other.offset
-            and self.symbol.uuid == other.symbol.uuid
-            and self.attributes == other.attributes
-        )
-
-    def __hash__(self):
-        # type: () -> int
-        return hash((self.offset, self.symbol.uuid))
-
-    def __repr__(self):
-        # type: () -> str
-        return (
-            "SymStackConst("
-            "offset={offset!r}, "
-            "symbol={symbol!r}, "
-            ")".format(**self.__dict__)
-        )
-
-    def deep_eq(self, other):
-        # type: (typing.Any) -> bool
-        # Do not move __eq__. See docstring for Node.deep_eq for more info.
-        if not isinstance(other, SymStackConst):
-            return False
-        return (
-            self.offset == other.offset
-            and self.symbol.deep_eq(other.symbol)
-            and self.attributes == other.attributes
-        )
+    @property
+    def symbols(self):
+        yield self.symbol
