@@ -2,7 +2,11 @@ import pkg_resources
 
 # Get the version from the package's version, so the version can be defined in
 # setup.py alone.
-API_VERSION = pkg_resources.require("gtirb")[0].version  # type: str
+API_VERSION = (
+    "1."
+    "10."
+    "4"
+)  # type: str
 """The semantic version of this API."""
 
 PROTOBUF_VERSION = 2  # type: int
